#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Drawing;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.IntegerVectorEncoding;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.PluginInfrastructure;
using HeuristicLab.Problems.GeneralizedQuadraticAssignment.Common;
using HeuristicLab.Problems.Instances;

namespace HeuristicLab.Problems.GeneralizedQuadraticAssignment {
  [Item("Generalized Quadratic Assignment Problem", "The Generalized Quadratic Assignment Problem (GQAP) is a generalization of the QAP in that it allows to assign multiple facilities (here called equipment) to a single location. The problem is described in Lee, C.G., and Ma, Z. 2003. The Generalized Quadratic Assignment Problem. Technical Report M5S 3G8, University of Toronto, Canada.")]
  [Creatable("Problems")]
  [StorableClass]
  public sealed class GeneralizedQuadraticAssignmentProblem : SingleObjectiveHeuristicOptimizationProblem<IGQAPEvaluator, IGQAPSolutionCreator>, IStorableContent,
    IConsumable<QAPInstance>,
    IConsumable<CTAPInstance>,
    IConsumable<TSPInstance>,
    IConsumable<ATSPInstance>,
    IConsumable<GQAPInstance> {

    public override Image ItemImage {
      get { return HeuristicLab.Common.Resources.VSImageLibrary.Type; }
    }

    public string Filename { get; set; }

    #region Parameter Descriptions
    public static readonly string MaximizationDescription = "False if the fitness function should be minimized (default) or otherwise True if it should be maximized.";
    public static readonly string WeightsDescription = "The weights matrix describes the flows between the equipments.";
    public static readonly string DistancesDescription = "The distances matrix describes the distances between the locations at which the equipment can be installed.";
    public static readonly string InstallationCostsDescription = "The installation costs matrix describes the installation costs of installing equipment i at location j";
    public static readonly string DemandsDescription = "The demands vector describes the space requirements of the equipments.";
    public static readonly string CapacitiesDescription = "The capacities vector describes the available space at the locations.";
    public static readonly string TransportationCostsDescription = "The transportation cost represents the flow-unit per distance-unit cost factor. This value can also be set to 1 if these costs are factored into the weights matrix already.";
    public static readonly string OverbookedCapacityPenaltyDescription = "The multiplier for the constraint violation when added to the quality.";
    public static readonly string BestKnownQualityDescription = "The best known quality (if available).";
    public static readonly string BestKnownSolutionDescription = "The best known solution (if available).";
    public static readonly string BestKnownSolutionsDescription = "Contains an archive of best-known solutions regarding flow-distance quality and installation quality.";
    public static readonly string EquipmentNamesDescription = "Optional: A list of names that describes the equipments.";
    public static readonly string LocationNamesDescription = "Optional: A list of names that describes the locations.";
    #endregion

    #region Parameter Properties
    public ValueParameter<DoubleMatrix> WeightsParameter {
      get { return (ValueParameter<DoubleMatrix>)Parameters["Weights"]; }
    }
    public ValueParameter<DoubleMatrix> DistancesParameter {
      get { return (ValueParameter<DoubleMatrix>)Parameters["Distances"]; }
    }
    public ValueParameter<DoubleMatrix> InstallationCostsParameter {
      get { return (ValueParameter<DoubleMatrix>)Parameters["InstallationCosts"]; }
    }
    public ValueParameter<DoubleArray> DemandsParameter {
      get { return (ValueParameter<DoubleArray>)Parameters["Demands"]; }
    }
    public ValueParameter<DoubleArray> CapacitiesParameter {
      get { return (ValueParameter<DoubleArray>)Parameters["Capacities"]; }
    }
    public FixedValueParameter<DoubleValue> TransportationCostsParameter {
      get { return (FixedValueParameter<DoubleValue>)Parameters["TransportationCosts"]; }
    }
    public FixedValueParameter<DoubleValue> OverbookedCapacityPenaltyParameter {
      get { return (FixedValueParameter<DoubleValue>)Parameters["OverbookedCapacityPenalty"]; }
    }
    public OptionalValueParameter<GQAPAssignment> BestKnownSolutionParameter {
      get { return (OptionalValueParameter<GQAPAssignment>)Parameters["BestKnownSolution"]; }
    }
    public OptionalValueParameter<GQAPAssignmentArchive> BestKnownSolutionsParameter {
      get { return (OptionalValueParameter<GQAPAssignmentArchive>)Parameters["BestKnownSolutions"]; }
    }
    public OptionalValueParameter<StringArray> EquipmentNamesParameter {
      get { return (OptionalValueParameter<StringArray>)Parameters["EquipmentNames"]; }
    }
    public OptionalValueParameter<StringArray> LocationNamesParameter {
      get { return (OptionalValueParameter<StringArray>)Parameters["LocationNames"]; }
    }
    #endregion

    #region Properties
    public DoubleMatrix Weights {
      get { return WeightsParameter.Value; }
      set { WeightsParameter.Value = value; }
    }
    public DoubleMatrix Distances {
      get { return DistancesParameter.Value; }
      set { DistancesParameter.Value = value; }
    }
    public DoubleMatrix InstallationCosts {
      get { return InstallationCostsParameter.Value; }
      set { InstallationCostsParameter.Value = value; }
    }
    public DoubleArray Demands {
      get { return DemandsParameter.Value; }
      set { DemandsParameter.Value = value; }
    }
    public DoubleArray Capacities {
      get { return CapacitiesParameter.Value; }
      set { CapacitiesParameter.Value = value; }
    }
    public DoubleValue TransportationCosts {
      get { return TransportationCostsParameter.Value; }
      set { TransportationCostsParameter.Value = value; }
    }
    public DoubleValue OverbookedCapacityPenalty {
      get { return TransportationCostsParameter.Value; }
      set { TransportationCostsParameter.Value = value; }
    }
    public StringArray EquipmentNames {
      get { return EquipmentNamesParameter.Value; }
      set { EquipmentNamesParameter.Value = value; }
    }
    public StringArray LocationNames {
      get { return LocationNamesParameter.Value; }
      set { LocationNamesParameter.Value = value; }
    }
    public GQAPAssignment BestKnownSolution {
      get { return BestKnownSolutionParameter.Value; }
      set { BestKnownSolutionParameter.Value = value; }
    }
    public GQAPAssignmentArchive BestKnownSolutions {
      get { return BestKnownSolutionsParameter.Value; }
      set { BestKnownSolutionsParameter.Value = value; }
    }
    #endregion

    public BestGQAPSolutionAnalyzer BestSolutionAnalyzer {
      get { return Operators.OfType<BestGQAPSolutionAnalyzer>().FirstOrDefault(); }
    }
    public GQAPSolutionArchiveAnalyzer SolutionArchiveAnalyzer {
      get { return Operators.OfType<GQAPSolutionArchiveAnalyzer>().FirstOrDefault(); }
    }

    [StorableConstructor]
    private GeneralizedQuadraticAssignmentProblem(bool deserializing) : base(deserializing) { }
    private GeneralizedQuadraticAssignmentProblem(GeneralizedQuadraticAssignmentProblem original, Cloner cloner)
      : base(original, cloner) {
      RegisterEventHandlers();
    }
    public GeneralizedQuadraticAssignmentProblem()
      : base(new GQAPEvaluator(), new RandomSolutionCreator()) {
      Parameters.Add(new ValueParameter<DoubleMatrix>("Weights", WeightsDescription, new DoubleMatrix(), false));
      Parameters.Add(new ValueParameter<DoubleMatrix>("Distances", DistancesDescription, new DoubleMatrix(), false));
      Parameters.Add(new ValueParameter<DoubleMatrix>("InstallationCosts", InstallationCostsDescription, new DoubleMatrix(), false));
      Parameters.Add(new FixedValueParameter<DoubleValue>("TransportationCosts", TransportationCostsDescription, new DoubleValue(1)));
      Parameters.Add(new FixedValueParameter<DoubleValue>("OverbookedCapacityPenalty", OverbookedCapacityPenaltyDescription, new DoubleValue(1000)));
      Parameters.Add(new ValueParameter<DoubleArray>("Demands", DemandsDescription, new DoubleArray(), false));
      Parameters.Add(new ValueParameter<DoubleArray>("Capacities", CapacitiesDescription, new DoubleArray(), false));
      Parameters.Add(new OptionalValueParameter<GQAPAssignment>("BestKnownSolution", BestKnownSolutionDescription, null, false));
      Parameters.Add(new OptionalValueParameter<GQAPAssignmentArchive>("BestKnownSolutions", BestKnownSolutionsDescription, null, false));
      Parameters.Add(new OptionalValueParameter<StringArray>("EquipmentNames", EquipmentNamesDescription, null, false));
      Parameters.Add(new OptionalValueParameter<StringArray>("LocationNames", LocationNamesDescription, null, false));

      WeightsParameter.ReactOnValueToStringChangedAndValueItemImageChanged = false;
      DistancesParameter.ReactOnValueToStringChangedAndValueItemImageChanged = false;
      InstallationCostsParameter.ReactOnValueToStringChangedAndValueItemImageChanged = false;
      DemandsParameter.ReactOnValueToStringChangedAndValueItemImageChanged = false;
      CapacitiesParameter.ReactOnValueToStringChangedAndValueItemImageChanged = false;

      Weights = new DoubleMatrix(5, 5);
      Weights[0, 0] = Weights[1, 1] = Weights[2, 2] = Weights[3, 3] = Weights[4, 4] = 0;
      Weights[0, 1] = Weights[1, 0] = Weights[0, 2] = Weights[2, 0] = Weights[0, 3] = Weights[3, 0] = Weights[0, 4] = Weights[4, 0] = 10;
      Weights[1, 2] = Weights[2, 1] = Weights[1, 3] = Weights[3, 1] = Weights[1, 4] = Weights[4, 1] = 5;
      Weights[2, 3] = Weights[3, 2] = Weights[2, 4] = Weights[4, 2] = 7.5;
      Weights[3, 4] = Weights[4, 3] = 2.5;

      Distances = new DoubleMatrix(3, 3);
      Distances[0, 0] = Distances[1, 1] = Distances[2, 2] = 0;
      Distances[0, 1] = Distances[1, 0] = Distances[1, 2] = Distances[2, 1] = 1;
      Distances[0, 2] = Distances[2, 0] = 2;

      InstallationCosts = new DoubleMatrix(5, 3);

      Demands = new DoubleArray(5);
      Demands[0] = 2; Demands[1] = 1; Demands[2] = 3; Demands[3] = 1; Demands[4] = 1;

      Capacities = new DoubleArray(3);
      Capacities[0] = 4; Capacities[1] = 1; Capacities[2] = 4;

      SolutionCreator.AssignmentParameter.ActualName = "Assignment";

      InitializeOperators();
      RegisterEventHandlers();
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new GeneralizedQuadraticAssignmentProblem(this, cloner);
    }

    #region Problem Instance Consumptions
    public bool LoadFrom(QAPInstance instance) {
      try {
        Name = instance.Name;
        Description = instance.Description;

        Weights = new DoubleMatrix(instance.Weights);
        Distances = new DoubleMatrix(instance.Distances);
        InstallationCosts = new DoubleMatrix(Weights.Rows, Distances.Columns); // all zero
        Capacities = new DoubleArray(Enumerable.Range(0, Distances.Rows).Select(x => 1.0).ToArray());
        Demands = new DoubleArray(Enumerable.Range(0, Weights.Rows).Select(x => 1.0).ToArray());

        TransportationCosts.Value = 1;

        if (instance.BestKnownAssignment != null) {
          EvaluateAndLoadAssignment(instance.BestKnownAssignment);
        } else {
          BestKnownQuality = null;
          BestKnownSolution = null;
          BestKnownSolutions = null;
        }
      } catch {
        return false;
      }
      return true;
    }

    public bool LoadFrom(CTAPInstance instance) {
      try {
        Name = instance.Name;
        Description = instance.Description;

        Capacities = new DoubleArray(instance.MemoryCapacities);
        Demands = new DoubleArray(instance.MemoryRequirements);
        Weights = new DoubleMatrix(instance.CommunicationCosts);
        InstallationCosts = new DoubleMatrix(instance.ExecutionCosts.Transpose());
        Distances = new DoubleMatrix(Capacities.Length, Capacities.Length);
        for (int i = 0; i < Capacities.Length - 1; i++)
          for (int j = i + 1; j < Capacities.Length; j++) {
            Distances[i, j] = 1;
          }

        TransportationCosts.Value = 1;

        if (instance.BestKnownAssignment != null) {
          EvaluateAndLoadAssignment(instance.BestKnownAssignment);
        } else {
          BestKnownQuality = null;
          BestKnownSolution = null;
          BestKnownSolutions = null;
        }
      } catch {
        return false;
      }
      return true;
    }

    public bool LoadFrom(TSPInstance instance) {
      try {
        if (instance.Dimension > 1000) return false;

        Name = instance.Name;
        Description = instance.Description;

        Capacities = new DoubleArray(instance.Dimension);
        Demands = new DoubleArray(instance.Dimension);
        for (int i = 0; i < instance.Dimension; i++) {
          Capacities[i] = 1;
          Demands[i] = 1;
        }
        InstallationCosts = new DoubleMatrix(instance.Dimension, instance.Dimension);
        Weights = new DoubleMatrix(instance.Dimension, instance.Dimension);
        for (int i = 0; i < instance.Dimension; i++)
          Weights[i, (i + 1) % instance.Dimension] = 1;
        Distances = new DoubleMatrix(instance.GetDistanceMatrix());

        TransportationCosts.Value = 1;

        if (instance.BestKnownTour != null) {
          EvaluateAndLoadAssignment(instance.BestKnownTour);
        } else {
          BestKnownQuality = null;
          BestKnownSolution = null;
          BestKnownSolutions = null;
        }
      } catch {
        return false;
      }
      return true;
    }

    public bool LoadFrom(ATSPInstance instance) {
      try {
        Name = instance.Name;
        Description = instance.Description;

        Capacities = new DoubleArray(instance.Dimension);
        Demands = new DoubleArray(instance.Dimension);
        for (int i = 0; i < instance.Dimension; i++) {
          Capacities[i] = 1;
          Demands[i] = 1;
        }
        InstallationCosts = new DoubleMatrix(instance.Dimension, instance.Dimension);
        Weights = new DoubleMatrix(instance.Dimension, instance.Dimension);
        for (int i = 0; i < instance.Dimension; i++)
          Weights[i, (i + 1) % instance.Dimension] = 1;
        Distances = new DoubleMatrix(instance.Distances);

        TransportationCosts.Value = 1;

        if (instance.BestKnownTour != null) {
          EvaluateAndLoadAssignment(instance.BestKnownTour);
        } else {
          BestKnownQuality = null;
          BestKnownSolution = null;
          BestKnownSolutions = null;
        }
      } catch {
        return false;
      }
      return true;
    }

    public bool LoadFrom(GQAPInstance instance) {
      try {
        Name = instance.Name;
        Description = instance.Description;

        Capacities = new DoubleArray(instance.Capacities);
        Demands = new DoubleArray(instance.Demands);
        InstallationCosts = new DoubleMatrix(instance.InstallationCosts);
        Weights = new DoubleMatrix(instance.Weights);
        Distances = new DoubleMatrix(instance.Distances);
        TransportationCosts.Value = instance.TransportationCosts;

        if (instance.BestKnownAssignment != null) {
          EvaluateAndLoadAssignment(instance.BestKnownAssignment);
        } else if (instance.BestKnownQuality.HasValue) {
          BestKnownQuality = new DoubleValue(instance.BestKnownQuality.Value);
          BestKnownSolution = null;
          BestKnownSolutions = null;
        } else {
          BestKnownQuality = null;
          BestKnownSolution = null;
          BestKnownSolutions = null;
        }
      } catch {
        return false;
      }
      return true;
    }
    #endregion

    private void EvaluateAndLoadAssignment(int[] vector) {
      EvaluateAndLoadAssignment(new IntegerVector(vector));
    }
    private void EvaluateAndLoadAssignment(IntegerVector assignment) {
      if (!IsConfigurationValid()) {
        BestKnownQuality = null;
        BestKnownSolution = null;
        BestKnownSolutions = null;
      }
      double flowDistanceQuality, installationQuality, overbookedCapacity;
      GQAPEvaluator.Evaluate(assignment, Weights, Distances, InstallationCosts, Demands, Capacities,
        out flowDistanceQuality, out installationQuality, out overbookedCapacity);
      double quality = GQAPEvaluator.GetCombinedQuality(flowDistanceQuality, installationQuality, overbookedCapacity, TransportationCosts.Value, OverbookedCapacityPenalty.Value);
      BestKnownSolution = new GQAPAssignment((IntegerVector)assignment.Clone(), new DoubleValue(quality),
        new DoubleValue(flowDistanceQuality), new DoubleValue(installationQuality),
        new DoubleValue(overbookedCapacity), EquipmentNames, LocationNames, Distances, Weights, InstallationCosts,
        Demands, Capacities, TransportationCosts, OverbookedCapacityPenalty);
      BestKnownQuality = new DoubleValue(quality);
      BestKnownSolutions = new GQAPAssignmentArchive(EquipmentNames, LocationNames, Distances, Weights, InstallationCosts, Demands, Capacities, TransportationCosts, OverbookedCapacityPenalty);
      BestKnownSolutions.Solutions.Add(new GQAPSolution((IntegerVector)assignment.Clone(), new DoubleValue(quality), new DoubleValue(flowDistanceQuality), new DoubleValue(installationQuality), new DoubleValue(overbookedCapacity)));
    }

    #region Events
    protected override void OnOperatorsChanged() {
      base.OnOperatorsChanged();
      Parameterize();
    }
    protected override void OnEvaluatorChanged() {
      base.OnEvaluatorChanged();
      Parameterize();
      Evaluator.QualityParameter.ActualNameChanged += new System.EventHandler(Evaluator_QualityParameter_ActualNameChanged);
    }
    protected override void OnSolutionCreatorChanged() {
      base.OnSolutionCreatorChanged();
      Parameterize();
      SolutionCreator.AssignmentParameter.ActualNameChanged += new EventHandler(SolutionCreator_IntegerVectorParameter_ActualNameChanged);
    }

    private void Evaluator_QualityParameter_ActualNameChanged(object sender, EventArgs e) {
      Parameterize();
    }
    private void SolutionCreator_IntegerVectorParameter_ActualNameChanged(object sender, EventArgs e) {
      Parameterize();
    }
    #endregion

    #region Helpers
    [StorableHook(HookType.AfterDeserialization)]
    private void AfterDeserialization() {
      RegisterEventHandlers();
    }

    private void RegisterEventHandlers() {
      Evaluator.QualityParameter.ActualNameChanged += new System.EventHandler(Evaluator_QualityParameter_ActualNameChanged);
      SolutionCreator.AssignmentParameter.ActualNameChanged += new EventHandler(SolutionCreator_IntegerVectorParameter_ActualNameChanged);
    }

    private void InitializeOperators() {
      Operators.Clear();
      Operators.AddRange(ApplicationManager.Manager.GetInstances<IGQAPOperator>());
      Operators.AddRange(ApplicationManager.Manager.GetInstances<IIntegerVectorOperator>());
      Operators.RemoveAll(x => x is ISingleObjectiveMoveEvaluator);
      Operators.AddRange(ApplicationManager.Manager.GetInstances<IGQAPMoveEvaluator>());
      Parameterize();
    }

    private void Parameterize() {

      var operators = Operators.Union(new IOperator[] { SolutionCreator, Evaluator }).ToArray();

      foreach (var op in operators.OfType<IAssignmentAwareGQAPOperator>()) {
        op.AssignmentParameter.ActualName = SolutionCreator.AssignmentParameter.ActualName;
      }
      foreach (var op in operators.OfType<IAssignmentsAwareGQAPOperator>()) {
        op.AssignmentParameter.ActualName = SolutionCreator.AssignmentParameter.ActualName;
      }
      foreach (var op in operators.OfType<IBestKnownQualityAwareGQAPOperator>()) {
        op.BestKnownQualityParameter.ActualName = BestKnownQualityParameter.Name;
      }
      foreach (var op in operators.OfType<IBestKnownSolutionAwareGQAPOperator>()) {
        op.BestKnownSolutionParameter.ActualName = BestKnownSolutionParameter.Name;
      }
      foreach (var op in operators.OfType<IBestKnownSolutionsAwareGQAPOperator>()) {
        op.BestKnownSolutionsParameter.ActualName = BestKnownSolutionsParameter.Name;
      }
      foreach (var op in operators.OfType<ICapacitiesAwareGQAPOperator>()) {
        op.CapacitiesParameter.ActualName = CapacitiesParameter.Name;
      }
      foreach (var op in operators.OfType<IDemandsAwareGQAPOperator>()) {
        op.DemandsParameter.ActualName = DemandsParameter.Name;
      }
      foreach (var op in operators.OfType<IDistancesAwareGQAPOperator>()) {
        op.DistancesParameter.ActualName = DistancesParameter.Name;
      }
      foreach (var op in operators.OfType<IEquipmentNamesAwareGQAPOperator>()) {
        op.EquipmentNamesParameter.ActualName = EquipmentNamesParameter.Name;
      }
      foreach (var op in operators.OfType<IGQAPCrossover>()) {
        op.ParentsParameter.ActualName = SolutionCreator.AssignmentParameter.ActualName;
        op.ChildParameter.ActualName = SolutionCreator.AssignmentParameter.ActualName;
      }
      foreach (var op in operators.OfType<IInstallationCostsAwareGQAPOperator>()) {
        op.InstallationCostsParameter.ActualName = InstallationCostsParameter.Name;
      }
      foreach (var op in operators.OfType<ILocationNamesAwareGQAPOperator>()) {
        op.LocationNamesParameter.ActualName = LocationNamesParameter.Name;
      }
      var moveEvaluator = operators.OfType<IGQAPMoveEvaluator>().FirstOrDefault();
      foreach (var op in operators.OfType<IGQAPMoveEvaluator>()) { // synchronize all move evaluators
        if (moveEvaluator != null) {
          op.MoveQualityParameter.ActualName = moveEvaluator.MoveQualityParameter.ActualName;
          op.MoveFlowDistanceQualityParameter.ActualName = moveEvaluator.MoveFlowDistanceQualityParameter.ActualName;
          op.MoveInstallationQualityParameter.ActualName = moveEvaluator.MoveInstallationQualityParameter.ActualName;
          op.MoveOverbookedCapacityParameter.ActualName = moveEvaluator.MoveOverbookedCapacityParameter.ActualName;
        }
      }
      foreach (var op in operators.OfType<IMoveQualityAwareGQAPOperator>()) {
        if (moveEvaluator != null) {
          op.MoveQualityParameter.ActualName = moveEvaluator.MoveQualityParameter.ActualName;
          op.MoveFlowDistanceQualityParameter.ActualName = moveEvaluator.MoveFlowDistanceQualityParameter.ActualName;
          op.MoveInstallationQualityParameter.ActualName = moveEvaluator.MoveInstallationQualityParameter.ActualName;
          op.MoveOverbookedCapacityParameter.ActualName = moveEvaluator.MoveOverbookedCapacityParameter.ActualName;
        }
        op.MaximizationParameter.ActualName = MaximizationParameter.Name;
      }
      foreach (var op in operators.OfType<IOverbookedCapacityPenaltyAwareGQAPOperator>()) {
        op.OverbookedCapacityPenaltyParameter.ActualName = OverbookedCapacityPenaltyParameter.Name;
      }
      foreach (var op in operators.OfType<IQualitiesAwareGQAPOperator>()) {
        op.QualityParameter.ActualName = Evaluator.QualityParameter.ActualName;
        op.FlowDistanceQualityParameter.ActualName = Evaluator.FlowDistanceQualityParameter.ActualName;
        op.InstallationQualityParameter.ActualName = Evaluator.InstallationQualityParameter.ActualName;
        op.OverbookedCapacityParameter.ActualName = Evaluator.OverbookedCapacityParameter.ActualName;
        op.MaximizationParameter.ActualName = MaximizationParameter.Name;
      }
      foreach (var op in operators.OfType<IQualityAwareGQAPOperator>()) {
        op.QualityParameter.ActualName = Evaluator.QualityParameter.ActualName;
        op.FlowDistanceQualityParameter.ActualName = Evaluator.FlowDistanceQualityParameter.ActualName;
        op.InstallationQualityParameter.ActualName = Evaluator.InstallationQualityParameter.ActualName;
        op.OverbookedCapacityParameter.ActualName = Evaluator.OverbookedCapacityParameter.ActualName;
        op.MaximizationParameter.ActualName = MaximizationParameter.Name;
      }
      foreach (var op in operators.OfType<ITransportationCostsAwareGQAPOperator>()) {
        op.TransportationCostsParameter.ActualName = TransportationCostsParameter.Name;
      }
      foreach (var op in operators.OfType<IWeightsAwareGQAPOperator>()) {
        op.WeightsParameter.ActualName = WeightsParameter.Name;
      }

      foreach (var op in operators.OfType<IIntegerVectorCrossover>()) {
        op.ParentsParameter.ActualName = SolutionCreator.AssignmentParameter.ActualName;
        op.ChildParameter.ActualName = SolutionCreator.AssignmentParameter.ActualName;
      }
      foreach (var op in operators.OfType<IIntegerVectorManipulator>()) {
        op.IntegerVectorParameter.ActualName = SolutionCreator.AssignmentParameter.ActualName;
      }
    }
    #endregion

    private bool IsConfigurationValid() {
      return Weights != null && Distances != null && Demands != null && Capacities != null && InstallationCosts != null
        && Weights.Rows == Weights.Columns && Weights.Rows == InstallationCosts.Rows
        && Distances.Rows == Distances.Columns && Distances.Rows == InstallationCosts.Columns
        && Demands.Length == Weights.Rows && Capacities.Length == Distances.Rows;
    }
  }
}
